<?php

namespace Drupal\animatecss_block\Service;

use Drupal\animatecss_block\Constants\AnimateCssBlockConstants;
use Drupal\animatecss_ui\AnimateCssManagerInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\Html;
use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityFormInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Session\AccountProxy;
use Drupal\Core\Url;
use Drupal\path_alias\AliasManagerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Block Class Service Class.
 */
class AnimateCssBlockHelperService {

  use StringTranslationTrait;

  /**
   * Animate manager.
   *
   * @var \Drupal\animatecss_ui\AnimateCssManagerInterface
   */
  protected $animateManager;

  /**
   * The UUID service.
   *
   * @var \Drupal\Component\Uuid\UuidInterface
   */
  protected $uuidService;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * Path Matcher.
   *
   * @var \Drupal\Core\Path\PathMatcherInterface
   */
  protected $pathMatcher;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The path alias manager.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * The current path.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPath;

  /**
   * Drupal\Core\Entity\EntityRepositoryInterface service.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $entityRepository;

  /**
   * Drupal\Core\Session\AccountProxy definition.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The block entity.
   *
   * @var useDrupal\block\Entity\Block
   */
  protected $blockEntity;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Construct of Block Class service.
   *
   * @param \Drupal\animatecss_ui\AnimateCssManagerInterface $animate_manager
   *   The Animate selector manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Core\Path\PathMatcherInterface $path_matcher
   *   The path matcher.
   * @param \Drupal\Component\Uuid\UuidInterface $uuid_service
   *   The UUID service.
   * @param \Drupal\path_alias\AliasManagerInterface $alias_manager
   *   The path alias manager.
   * @param \Drupal\Core\Path\CurrentPathStack $current_path
   *   The current path.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entityRepository
   *   The entity repository.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   */
  public function __construct(AnimateCssManagerInterface $animate_manager, TimeInterface $time, LanguageManagerInterface $language_manager, ConfigFactoryInterface $config_factory, Connection $database, RequestStack $request_stack, PathMatcherInterface $path_matcher, UuidInterface $uuid_service, AliasManagerInterface $alias_manager, CurrentPathStack $current_path, EntityRepositoryInterface $entityRepository, AccountProxy $currentUser, EntityTypeManagerInterface $entity_manager) {
    $this->animateManager = $animate_manager;
    $this->time = $time;
    $this->languageManager = $language_manager;
    $this->pathMatcher = $path_matcher;
    $this->request = $request_stack->getCurrentRequest();
    $this->configFactory = $config_factory;
    $this->database = $database;
    $this->uuidService = $uuid_service;
    $this->aliasManager = $alias_manager;
    $this->currentPath = $current_path;
    $this->entityRepository = $entityRepository;
    $this->currentUser = $currentUser;
    $this->entityTypeManager = $entity_manager;
    $this->blockEntity = $this->entityTypeManager->getStorage('block');
  }

  /**
   * Method to do the presave block.
   */
  public function animateCssBlockPreSave(&$entity) {
    $animatecss_status = $entity->getThirdPartySetting('animatecss_block', 'status');

    // If AnimateCSS is not enabled in the block, unset the ThirdPartySetting.
    if (!$animatecss_status) {
      $entity->unsetThirdPartySetting('animatecss_block', 'status');
      $entity->unsetThirdPartySetting('animatecss_block', 'animation');
      $entity->unsetThirdPartySetting('animatecss_block', 'delay');
      $entity->unsetThirdPartySetting('animatecss_block', 'time');
      $entity->unsetThirdPartySetting('animatecss_block', 'speed');
      $entity->unsetThirdPartySetting('animatecss_block', 'duration');
      $entity->unsetThirdPartySetting('animatecss_block', 'repeat');
    }
    else {
      // Get the config object.
      $config = $this->configFactory->getEditable('animatecss_block.settings');

      // Get the AnimateCSS block ThirdPartySetting.
      $settings = [
        'animation' => $entity->getThirdPartySetting('animatecss_block', 'animation'),
        'delay'     => $entity->getThirdPartySetting('animatecss_block', 'delay'),
        'time'      => $entity->getThirdPartySetting('animatecss_block', 'time'),
        'speed'     => $entity->getThirdPartySetting('animatecss_block', 'speed'),
        'duration'  => $entity->getThirdPartySetting('animatecss_block', 'duration'),
        'repeat'    => $entity->getThirdPartySetting('animatecss_block', 'repeat'),
      ];

      // Set the AnimateCSS block ThirdPartySetting.
      $entity->setThirdPartySetting('animatecss_block', 'status', $animatecss_status);
      $entity->setThirdPartySetting('animatecss_block', 'animation', $settings['animation']);
      $entity->setThirdPartySetting('animatecss_block', 'delay', $settings['delay']);
      $entity->setThirdPartySetting('animatecss_block', 'time', $settings['time']);
      $entity->setThirdPartySetting('animatecss_block', 'speed', $settings['speed']);
      $entity->setThirdPartySetting('animatecss_block', 'duration', $settings['duration']);
      $entity->setThirdPartySetting('animatecss_block', 'repeat', $settings['repeat']);

      // Get the current AnimateCSS blocks stored.
      $animatecss_blocks_stored = $config->get('blocks');

      // Get the current block ID.
      $animatecss_block_id = $entity->id();

      // Merge with the current one.
      array_push($animatecss_blocks_stored, $animatecss_block_id);
      $animatecss_blocks_to_store = array_unique($animatecss_blocks_stored);

      // Store in the config.
      $config->set('blocks', $animatecss_blocks_to_store);

      // Save.
      $config->save();

      if (!$this->animateManager->isAnimate($animatecss_block_id)) {
        $animate_id = 0;

        $selector = '#block-' . strtr($animatecss_block_id, '_', '-');
        $label    = 'AnimateCSS Block - ' . ucfirst(trim(preg_replace("/[^a-zA-Z0-9]+/", " ", $animatecss_block_id)));
        $comment  = 'This AnimateCSS related to block configuration.';

        // Set variables from main animate settings.
        $variables['animation'] = $settings['animation'];
        $variables['delay']     = $settings['delay'];
        $variables['time']      = $settings['time'];
        $variables['speed']     = $settings['speed'];
        $variables['duration']  = $settings['duration'];
        $variables['repeat']    = $settings['repeat'];

        // Get variables from other module.
        /*if (isset($values['variables']) && count($values['variables'])) {
          $variables = $variables + $values['variables'];
        }
        if (count($this->variables)) {
          $variables = array_merge($variables, $this->variables);
        }*/

        // Serialize options variables.
        $options = serialize($variables);

        // The Unix timestamp when the animate was most recently saved.
        $changed = $this->time->getCurrentTime();

        // Save animate.
        $this->animateManager->addAnimate($animate_id, $selector, $label, $comment, $changed, $animatecss_status, $options);

        // Flush caches so the updated config can be checked.
        //drupal_flush_all_caches();
      }
    }

  }

  /**
   * Method to preprocess block.
   */
  public function animateCssBlockPreprocessBlock(&$variables) {
    // Blocks coming from page manager widget does not have id.
    // If there is no Block ID, skip that.
    /*if (empty($variables['elements']['#id'])) {
      return;
    }

    // If there is no block with this ID, skip.
    if (empty($this->blockEntity->load($variables['elements']['#id']))) {
      return;
    }

    // Load the block by ID.
    $block = $this->blockEntity->load($variables['elements']['#id']);

    // Add attributes on block.
    $this->addAttributesOnBlock($block, $variables);

    // Add classes on block.
    $this->addClassesOnBlock($block, $variables);

    // Add the new ID on block.
    $this->updateBlockId($block, $variables);*/

  }

  /**
   * Method to add attributes on block.
   */
  public function addAttributesOnBlock(&$block, &$variables) {

    // Get the config object.
    $config = $this->configFactory->getEditable('animatecss_block.settings');

    // If attributes isn't enabled, skip.
    if (empty($config->get('enable_attributes'))) {
      return FALSE;
    }

    // If there is no attributes on block, skip.
    if (empty($block->getThirdPartySetting('animatecss_block', 'attributes'))) {
      return FALSE;
    }

    $attributes = $block->getThirdPartySetting('animatecss_block', 'attributes');

    $attributes = explode(PHP_EOL, $attributes);

    foreach ($attributes as $attribute) {

      $attribute = explode('|', $attribute);

      $attribute_key = trim($attribute[0]);

      $attribute_value = trim($attribute[1]);

      // Insert the attributes.
      $variables['attributes'][$attribute_key][] = $attribute_value;
    }
  }

  /**
   * Method to update the ID on block.
   */
  public function updateBlockId(&$block, &$variables) {

    // If there is no replaced id on block, skip.
    if (empty($block->getThirdPartySetting('animatecss_block', 'replaced_id'))) {
      return FALSE;
    }

    // Verify if the block has Third Party Settings with replaced id.
    $replaced_id = $block->getThirdPartySetting('animatecss_block', 'replaced_id');

    // Remove extra spaces.
    $replaced_id = trim($replaced_id);

    // If the user selected <none> in the block settings disable the block id.
    if ($replaced_id == '<none>') {
      unset($variables['attributes']['id']);
      return;
    }

    // Update the ID.
    $variables['attributes']['id'] = $replaced_id;

  }

  /**
   * Method to add classes on block.
   */
  public function addClassesOnBlock(&$block, &$variables) {

    // Verify if the current block has Third Party Settings with classes.
    $classes = $block->getThirdPartySetting('animatecss_block', 'classes');

    if (empty($classes)) {
      return FALSE;
    }

    // Get all classes if exists.
    $classes_array = explode(' ', $classes);

    // Get the config object.
    $config = $this->configFactory->getEditable('animatecss_block.settings');

    $filter_css_identifier = [];

    if (!empty($config->get('filter_html_clean_css_identifier'))) {

      $filter_clean_css_identifier = $config->get('filter_html_clean_css_identifier');

      $filter_clean_css_identifier = explode(PHP_EOL, $filter_clean_css_identifier);

      foreach ($filter_clean_css_identifier as $filter) {

        $filter = explode('|', $filter);

        $origin_char = trim($filter[0]);
        $target_char = trim($filter[1]);

        $filter_css_identifier[$origin_char] = $target_char;

      }

    }

    // Add all classes.
    foreach ($classes_array as $class) {
      // Skip cleanCssIdentifier if the option "Allow Special Char" is enabled.
      if ($config->get('enable_special_chars') == FALSE) {
        $class = Html::cleanCssIdentifier($class, $filter_css_identifier);
      }
      $variables['attributes']['class'][] = $class;
    }
  }

  /**
   * Method to do a form alter.
   */
  public function animateCssBlockFormAlter(&$form, &$form_state) {
    // If the user don't have permission, skip that.
    if (!$this->currentUser->hasPermission(AnimateCssBlockConstants::ANIMATECSS_BLOCK_PERMISSION)) {
      return;
    }

    $form_object = $form_state->getFormObject();

    // Implement the alter only if is an instance of EntityFormInterface.
    if (!($form_object instanceof EntityFormInterface)) {
      return;
    }

    /** @var \Drupal\block\BlockInterface $block */
    $block = $form_object->getEntity();

    // Load the Animate.css configuration settings.
    $config_animatecss = $this->configFactory->getEditable('animatecss.settings');

    // Attach Animate.css to pages with chosen method.
    $method = animatecss_check_installed() ? $config_animatecss->get('method') : 'cdn';
    $compat = $config_animatecss->get('compat');

    // Attach Animate.css for examples.
    $sample = $config_animatecss->get('options');
    $sample['selector'] = '.animate__sample';

    $form['#attached']['drupalSettings']['animatecss']['sample'] = $sample;
    $form['#attached']['drupalSettings']['animatecss']['compat'] = $compat;

    if ($method == 'cdn') {
      $form['#attached']['library'][] = 'animatecss_ui/animate-cdn-dev';
    }
    else {
      $form['#attached']['library'][] = 'animatecss_ui/animate-dev';
    }

    // Attach AnimateCSS form library.
    $form['#attached']['library'][] = 'animatecss_ui/animate-form';

    // Get AnimateCSS Block config object.
    $config = $this->configFactory->getEditable('animatecss_block.settings');

    // Get AnimateCSS block settings default value.
    $options = [
      'status'    => $block->getThirdPartySetting('animatecss_block', 'status', $config->get('status')),
      'animation' => $block->getThirdPartySetting('animatecss_block', 'animation', $config_animatecss->get('options.animation')),
      'delay'     => $block->getThirdPartySetting('animatecss_block', 'delay', $config_animatecss->get('options.delay')),
      'time'      => $block->getThirdPartySetting('animatecss_block', 'time', $config_animatecss->get('options.time')),
      'speed'     => $block->getThirdPartySetting('animatecss_block', 'speed', $config_animatecss->get('options.speed')),
      'duration'  => $block->getThirdPartySetting('animatecss_block', 'duration', $config_animatecss->get('options.duration')),
      'repeat'    => $block->getThirdPartySetting('animatecss_block', 'repeat', $config_animatecss->get('options.repeat')),
    ];

    // Get the URL settings global page.
    $url_settings_page = Url::fromRoute('animatecss_block.settings')->toString();

    $url_used_items_list = Url::fromRoute('animatecss_block.class_list')->toString();

    // Put the default help text.
    $help_text = $this->t('Customize the styling of this block by adding CSS classes.');

    // Put the Modal with all items used.
    $help_text .= ' <div class="show-items-used">' . $this->t('<a href="@url_used_items_list@" class="use-ajax" data-dialog-options="{&quot;width&quot;:800}" data-dialog-type="modal">See all the classes used</a>.</div>', [
      '@url_used_items_list@' => $url_used_items_list,
    ]);

    $form['animatecss'] = [
      '#type'        => 'details',
      '#title'       => $this->t('AnimateCSS'),
      '#open'        => TRUE,
      '#description' => $help_text,
      '#weight'      => 0,
    ];

    // Put the weight only if exists.
    if (!empty($config->get('weight')) || $config->get('weight') !== '0') {
      $form['animatecss']['#weight'] = $config->get('weight');
    }

    // This will automatically be saved in the third party settings.
    $form['animatecss']['third_party_settings']['#tree'] = TRUE;

    // Enabled status for this animate.
    $form['animatecss']['third_party_settings']['animatecss_block']['status'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t('Enabled'),
      '#default_value' => $options['status'] ?? FALSE,
    ];

    // AnimateCSS fields wrapper.
    $form['animatecss']['third_party_settings']['animatecss_block']['container'] = [
      '#type'   => 'container',
      '#states' => [
        'visible' => [
          'input[name="animatecss[third_party_settings][animatecss_block][status]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // The animation to use.
    $form['animatecss']['third_party_settings']['animatecss_block']['container']['animation'] = [
      '#title'         => $this->t('Animation'),
      '#type'          => 'select',
      '#options'       => animatecss_animation_options(),
      '#default_value' => $options['animation'] ?? $config_animatecss->get('options.animation'),
      '#description'   => $this->t('Select the animation name you want to use for CSS selectors globally.'),
      '#attributes'    => ['class' => ['animate__animation']],
    ];

    // Animate.css provides the following delays.
    // Animate duration used as a prefix on CSS Variables.
    $form['animatecss']['third_party_settings']['animatecss_block']['container']['delay_wrapper'] = [
      '#type'       => 'container',
      '#attributes' => [
        'class' => ['animatecss-wrapper', 'container-inline', 'form-item'],
      ],
    ];
    $form['animatecss']['third_party_settings']['animatecss_block']['container']['delay_wrapper']['delay'] = [
      '#title'         => $this->t('Delay'),
      '#type'          => 'select',
      '#options'       => animatecss_delay_options(),
      '#default_value' => $options['delay'] ?? $config_animatecss->get('options.delay'),
    ];
    $form['animatecss']['third_party_settings']['animatecss_block']['container']['delay_wrapper']['time'] = [
      '#type'          => 'number',
      '#min'           => 0,
      '#title'         => $this->t('Delay time'),
      '#default_value' => $options['time'] ?? $config_animatecss->get('options.time'),
      '#field_suffix'  => 'ms',
      '#attributes'    => ['class' => ['animate-delay-time']],
      '#states'        => [
        'visible' => [
          'select[name="animatecss[third_party_settings][animatecss_block][container][delay_wrapper][delay]"]' => ['value' => 'custom'],
        ],
      ],
    ];
    $form['animatecss']['third_party_settings']['animatecss_block']['container']['delay_wrapper']['time_description'] = [
      '#type'   => 'markup',
      '#markup' => $this->t('The provided delays are from 1 to 5 seconds. You can customize them by selecting custom and set the delay time directly on the elements to delay the start of the animation.'),
      '#prefix' => '<div class="form-item__description">',
      '#suffix' => '</div>',
    ];

    // Animate speed time and duration used as a prefix on CSS Variables.
    $form['animatecss']['third_party_settings']['animatecss_block']['container']['duration_wrapper'] = [
      '#type'       => 'container',
      '#attributes' => [
        'class' => ['animatecss-wrapper', 'container-inline', 'form-item'],
      ],
    ];
    $form['animatecss']['third_party_settings']['animatecss_block']['container']['duration_wrapper']['speed'] = [
      '#title'         => $this->t('Speed'),
      '#type'          => 'select',
      '#options'       => animatecss_speed_options(),
      '#default_value' => $options['speed'] ?? $config_animatecss->get('options.speed'),
    ];
    $form['animatecss']['third_party_settings']['animatecss_block']['container']['duration_wrapper']['duration'] = [
      '#type'          => 'number',
      '#title'         => $this->t('Duration'),
      '#default_value' => $options['duration'] ?? $config_animatecss->get('options.duration'),
      '#field_suffix'  => 'ms',
      '#attributes'    => ['class' => ['animate-duration']],
      '#states'        => [
        'visible' => [
          'select[name="animatecss[third_party_settings][animatecss_block][container][duration_wrapper][speed]"]' => ['value' => 'custom'],
        ],
      ],
    ];
    $form['animatecss']['third_party_settings']['animatecss_block']['container']['duration_wrapper']['speed_description'] = [
      '#type'   => 'markup',
      '#markup' => $this->t('You can control the speed of the animation. The medium option speed is 1 second same as a default speed. You can also set the animations duration by selecting customize option.'),
      '#prefix' => '<div class="form-item__description">',
      '#suffix' => '</div>',
    ];

    // Animate iteration count.
    $form['animatecss']['third_party_settings']['animatecss_block']['container']['repeat'] = [
      '#title'         => $this->t('Repeating'),
      '#type'          => 'select',
      '#options'       => animatecss_repeat_options(),
      '#default_value' => $options['repeat'] ?? $config_animatecss->get('options.repeat'),
      '#description'   => $this->t('You can control the iteration count of the animation.'),
    ];

    $form['#validate'][] = 'animatecss_block_form_block_form_validate';
  }

  /**
   * Method to do a form validate.
   */
  public function animateCssBlockFormValidate(&$form, &$form_state) {
    // Get the ThirdPartySettings.
    $animatecss = $form_state->getValue('animatecss') ? $form_state->getValue('animatecss')['third_party_settings']['animatecss_block'] : [];

    // Clear empty values.
    if (count($animatecss)) {
      $animatecss_block = [
        'status'    => $animatecss['status'],
        'animation' => $animatecss['container']['animation'],
        'delay'     => $animatecss['container']['delay_wrapper']['delay'],
        'time'      => $animatecss['container']['delay_wrapper']['time'],
        'speed'     => $animatecss['container']['duration_wrapper']['speed'],
        'duration'  => $animatecss['container']['duration_wrapper']['duration'],
        'repeat'    => $animatecss['container']['repeat'],
      ];
      $third_party_settings['animatecss_block'] = array_filter($animatecss_block);
    }

    // Set the ThirdPartySettings with the default array.
    $form_state->setValue('third_party_settings', $third_party_settings);
  }

}
